package cn.fw.valhalla.rpc.erp;

import cn.fw.data.base.domain.common.Message;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.valhalla.rpc.AbsBaseRpcService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.erp.dto.UserSimpleDTO;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 用户服务
 * <p>
 * create at 2019-03-09
 *
 * @author kurisu
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class UserService extends AbsBaseRpcService {
    /**
     * 岗位用户服务
     */
    private final UserRoleApi userRoleApi;

    @Value("${spring.cache.custom.global-prefix}:rpc:erp")
    @Getter
    private String keyPrefix;


    /**
     * 获取某个流程角色的权限范围
     *
     * @param userId
     * @param roleCode
     * @return
     */
    public List<UserRoleDataRangeDTO> getUserRoleDataRange(final Long userId, final String roleCode) {
        if (Objects.isNull(userId) || Objects.isNull(roleCode)) {
            return Collections.emptyList();
        }
        String key = generateKey(roleCode, userId);
        try {
            List<UserRoleDataRangeDTO> dtos = getListFromCache(key, UserRoleDataRangeDTO.class);
            if (!CollectionUtils.isEmpty(dtos)) {
                return dtos;
            }
            Message<List<UserRoleDataRange>> msg = userRoleApi.queryUserRoleDataRange(userId, roleCode);
            log.info("userRoleApi.queryUserRoleDataRange: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            List<UserRoleDataRange> data = msg.getData();
            List<UserRoleDataRangeDTO> list = new ArrayList<>(data.size());
            for (UserRoleDataRange datum : data) {
                UserRoleDataRangeDTO dto = new UserRoleDataRangeDTO();
                BeanUtils.copyProperties(datum, dto);
                list.add(dto);
                setListToCache(key, JSONObject.toJSONString(dto));
            }
            getQueue(key).expire(15, TimeUnit.SECONDS);
            return list;
        } catch (Exception e) {
            log.error("获取某个流程角色的权限范围失败：", e);
        }
        return Collections.emptyList();
    }

    /**
     * 获取门店下某岗位下员工
     *
     * @param shopId   门店
     * @param roleCode 岗位码
     * @return 用户集合
     */
    public List<PostUserDTO> getUserByRole(final Long shopId, final String roleCode) {
        if (Objects.isNull(shopId) || Objects.isNull(roleCode)) {
            return Collections.emptyList();
        }
        String key = generateKey(shopId, roleCode);
        List<PostUserDTO> dtos = getListFromCache(key, PostUserDTO.class);
        if (!CollectionUtils.isEmpty(dtos)) {
            return dtos;
        }
        try {
            log.info("查询门店[{}]下的[{}]角色", shopId, roleCode);
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(roleCode, shopId);
            log.info("userRoleApi.queryUserRoleInfos: msg.code={}, msg.result={}, msg.data={}", msg.getCode(), msg.getResult(), msg.getData());
            if (!msg.isSuccess() || CollectionUtils.isEmpty(msg.getData())) {
                return Collections.emptyList();
            }
            List<UserRoleInfo> data = msg.getData();
            List<PostUserDTO> list = new ArrayList<>(data.size());
            for (UserRoleInfo datum : data) {
                PostUserDTO dto = new PostUserDTO();
                BeanUtils.copyProperties(datum, dto);
                list.add(dto);
                setListToCache(key, JSONObject.toJSONString(dto));
            }
            getQueue(key).expire(10, TimeUnit.MINUTES);
            return list;
        } catch (Exception e) {
            log.error("获取门店下某岗位下员工失败：", e);
        }
        return Collections.emptyList();
    }

    /**
     * 门店内任意满足其中一个角色的人员
     *
     * @param shopId
     * @param roleCode
     * @return
     */
    public List<PostUserDTO> getShopRolesUser(Long shopId, String... roleCode) {
        if (roleCode.length == 0 || Objects.isNull(shopId)) {
            return Collections.emptyList();
        }
        String key = generateKey(shopId, String.join("|", roleCode));
        List<PostUserDTO> dtos = getListFromCache(key, PostUserDTO.class);
        if (!CollectionUtils.isEmpty(dtos)) {
            return dtos;
        }
        try {
            List<String> roleCodeList = Arrays.stream(roleCode).collect(Collectors.toList());
            Message<List<UserRoleInfo>> msg = userRoleApi.queryRoleUsers(roleCodeList, shopId);
            List<UserRoleInfo> userRoleInfos = msg.getData();
            if (!msg.isSuccess() || CollectionUtils.isEmpty(userRoleInfos)) {
                return Collections.emptyList();
            }
            List<PostUserDTO> list = new ArrayList<>(userRoleInfos.size());
            for (UserRoleInfo datum : userRoleInfos) {
                PostUserDTO dto = new PostUserDTO();
                BeanUtils.copyProperties(datum, dto);
                list.add(dto);
                setListToCache(key, JSONObject.toJSONString(dto));
            }
            getQueue(key).expire(10, TimeUnit.MINUTES);
            return list;
        } catch (Exception e) {
            log.error("查询任意满足条件的角色失败：", e);
        }
        return Collections.emptyList();
    }

    /**
     * 查询同时满足角色的人员
     *
     * @param shopId   门店
     * @param roleCode 角色码
     * @return
     */
    public List<UserSimpleDTO> queryShopRolesUser(Long shopId, String... roleCode) {
        final int length = roleCode.length;
        if (length == 0 || Objects.isNull(shopId)) {
            return Collections.emptyList();
        }
        final String key = generateKey(shopId, String.join("&", roleCode));
        List<UserSimpleDTO> dtos = getListFromCache(key, UserSimpleDTO.class);
        if (!CollectionUtils.isEmpty(dtos)) {
            return dtos;
        }
        try {
            final Set<UserSimpleDTO> users = new HashSet<>();
            final ConcurrentHashMap<Long, Integer> userCountMap = new ConcurrentHashMap<>();
            for (String s : roleCode) {
                Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(s, shopId);
                List<UserRoleInfo> userRoleInfos = msg.getData();
                if (!msg.isSuccess() || CollectionUtils.isEmpty(userRoleInfos)) {
                    return Collections.emptyList();
                }
                for (UserRoleInfo roleInfo : userRoleInfos) {
                    users.add(new UserSimpleDTO(roleInfo.getUserId(), roleInfo.getUserName()));
                    Integer count = userCountMap.getOrDefault(roleInfo.getUserId(), 0);
                    userCountMap.put(roleInfo.getUserId(), count + 1);
                }
            }
            final Set<Long> matchUserIds = userCountMap.entrySet().stream()
                    .filter(r -> r.getValue().equals(length))
                    .map(Map.Entry::getKey)
                    .collect(Collectors.toSet());

            if (CollectionUtils.isEmpty(matchUserIds)) {
                return Collections.emptyList();
            }
            final List<UserSimpleDTO> matchUserList = users.stream().filter(r -> matchUserIds.contains(r.getUserId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(matchUserList)) {
                return Collections.emptyList();
            }
            for (UserSimpleDTO simpleDTO : matchUserList) {
                setListToCache(key, JSONObject.toJSONString(simpleDTO));
            }
            getQueue(key).expire(10, TimeUnit.MINUTES);
            return matchUserList;
        } catch (Exception e) {
            log.error("查询同时满足条件的角色失败：", e);
        }
        return Collections.emptyList();
    }

    private String generateKey(final Long shopId, final String roleCOde) {
        Assert.notNull(shopId, "id cannot be null");
        Assert.hasLength(roleCOde, "roleCOde cannot be null");

        return String.format("%s:%s:%s:%s", getKeyPrefix(), "post", shopId, roleCOde);
    }

    private String generateKey(final String roleCode, final Long userId) {
        Assert.notNull(userId, "id cannot be null");
        Assert.hasLength(roleCode, "roleCOde cannot be null");

        return String.format("%s:%s:%s:%s", getKeyPrefix(), "range", roleCode, userId);
    }

    private String generateKey(final Long userId) {
        Assert.notNull(userId, "userId cannot be null");

        return String.format("%s:%s:%s", getKeyPrefix(), "user", userId);
    }
}
