package cn.fw.valhalla.rpc.member;

import cn.fw.common.util.MobileUtil;
import cn.fw.common.util.ValidationUtils;
import cn.fw.data.base.domain.common.Message;
import cn.fw.member.sdk.api.AccountApi;
import cn.fw.member.sdk.api.FunctionApi;
import cn.fw.member.sdk.api.MemberApi;
import cn.fw.member.sdk.vo.AccountInfoVo;
import cn.fw.member.sdk.vo.MemberInfoVo;
import cn.fw.member.sdk.vo.MemberPhoneInfoVo;
import cn.fw.member.sdk.vo.MobileLocation;
import cn.fw.valhalla.rpc.member.dto.MemberUserDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.Objects;

import static org.apache.commons.lang3.Validate.isTrue;

/**
 * 会员服务
 *
 * @author kurisu
 */
@Slf4j
@Service
public class MemberRpcService {
    /**
     * 会员服务
     */
    private final MemberApi memberApi;
    private final AccountApi accountApi;
    private final FunctionApi functionApi;

    /**
     * 默认构造器
     */
    @Autowired
    public MemberRpcService(final MemberApi memberApi,
                            final AccountApi accountApi,
                            final FunctionApi functionApi) {
        this.memberApi = memberApi;
        this.accountApi = accountApi;
        this.functionApi = functionApi;
    }

    /**
     * 根据会员ID获取会员信息
     *
     * @param userId 会员ID
     * @return 会员信息
     */
    public MemberUserDTO user(final Long userId) {
        if (userId == null || userId <= 0) {
            return null;
        }
        try {
            final Message<MemberPhoneInfoVo> msg = memberApi.getMemberPhoneById(userId);
            if (!msg.isSuccess()) {
                log.warn("调用Member[根据会员ID[{}]获取会员信息]系统失败, 原因：{}", userId, msg.getResult());
                return null;
            }
            final MemberPhoneInfoVo user = msg.getData();
            if (user != null) {
                final MemberUserDTO userDTO = new MemberUserDTO();
                BeanUtils.copyProperties(user, userDTO);
                return userDTO;
            }
        } catch (Exception e) {
            log.error("调用Member[根据会员ID[{}]获取会员信息]系统失败", userId, e);
        }
        return null;
    }

    /**
     * 根据手机号查询会员信息
     *
     * @param mobile 手机号
     * @return 会员信息
     */
    public MemberUserDTO queryByMobile(final String mobile) {
        if (StringUtils.isBlank(mobile)) {
            return null;
        }
        try {
            Message<AccountInfoVo> accountMsg = accountApi.getAccountByPhone(mobile);
            isTrue(accountMsg.isSuccess(), String.format("调用Member系统失败: [根据会员手机号[%s]获取会员信息: %s]", mobile, accountMsg.getResult()));
            AccountInfoVo account = accountMsg.getData();
            if (account == null) {
                log.error("手机号[{}]不存在任何账号信息", mobile);
                return null;
            }
            final Message<MemberInfoVo> msg = memberApi.getMemberById(account.getMemberId());
            isTrue(msg.isSuccess(), String.format("调用Member系统失败: [根据会员手机号[%s]获取会员信息L %s]", mobile, msg.getResult()));
            final MemberInfoVo user = msg.getData();
            if (Objects.isNull(user)) {
                return null;
            }
            final MemberUserDTO userDTO = new MemberUserDTO();
            BeanUtils.copyProperties(user, userDTO);
            userDTO.setLatestAccountId(account.getId());
            userDTO.setLatestAccountPhone(account.getPhone());
            return userDTO;
        } catch (Exception e) {
            log.error("调用Member系统根据会员手机号[{}]获取会员信息失败", mobile, e);
        }
        return null;
    }

    /**
     * 查询手机号归属地
     *
     * @param mobile
     * @return
     */
    @Cacheable(cacheNames = "mobile:attribution", key = "#mobile", unless = "#result.isEmpty()")
    public String attribution(final String mobile) {
        if (StringUtils.isBlank(mobile) || !ValidationUtils.checkMobile(mobile)) {
            return "";
        }
        try {
            MobileUtil.Result result = MobileUtil.attribution(mobile);
            String att = "";
            if (Objects.nonNull(result)) {
                att = result.getProvince().concat(" ").concat(result.getCity());
            }

            if (!StringUtils.isBlank(att.trim())) {
                return att.trim();
            }

            Message<MobileLocation> msg = functionApi.queryMobileLocation(mobile);
            if (!msg.isSuccess()) {
                return "";
            }
            MobileLocation data = msg.getData();
            if (data != null) {
                String province = data.getProvince();
                String city = data.getCity();
                att = province.concat(" ").concat(city);
            }
            return att.trim();
        } catch (Exception e) {
            log.warn("查询手机号归属地失败：", e);
        }
        return "";
    }

}
