package cn.fw.valhalla.sdk.api;

import cn.fw.data.base.domain.common.Message;
import cn.fw.valhalla.sdk.param.*;
import cn.fw.valhalla.sdk.result.AccidentFollowerResult;
import cn.fw.valhalla.sdk.result.CustomerClueDeadline;
import cn.fw.valhalla.sdk.result.CustomerSimpleInfoDto;
import cn.fw.valhalla.sdk.result.InboundRateTargetResult;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import java.util.List;

/**
 * @author : kurisu
 * @className : ValhallaGeneralApiService
 * @description : 售后crm通用api
 * @date: 2020-11-11 15:33
 */
@FeignClient(value = "fw-valhalla", path = "/api/valhalla/general")
public interface ValhallaGeneralApiService {
    /**
     * 保存进站记录
     *
     * @param reachLogReq
     * @return
     */
    @PostMapping("/reach/save")
    Message<Void> saveReachLog(@Valid @RequestBody ReachLogReq reachLogReq);

    /**
     * 查询保有客档案信息
     *
     * @param customerQueryReq
     * @return
     */
    @PostMapping("/query/customer/list")
    Message<List<CustomerSimpleInfoDto>> queryCustomList(@RequestBody CustomerQueryReq customerQueryReq);

    /**
     * 查询车辆最短的一个跟进的截止日期（不包含事故车）
     *
     * @param vin
     * @param groupId
     * @return
     */
    @GetMapping("/query/customer/clue/deadline")
    Message<CustomerClueDeadline> queryClueDeadline(@RequestParam("vin") String vin, @RequestParam("groupId") Long groupId);

    /**
     * 查询车辆最短的一个跟进的截止日期（不包含事故车）
     *
     * @param params
     * @return
     */
    @PostMapping("/query/customer/clue/batch/deadline")
    Message<List<CustomerClueDeadline>> queryClueDeadlineBatch(@Valid @RequestBody ClueDeadlineParams params);

    /**
     * 根据车牌号查询事故车跟进人员
     *
     * @param plateNo 车牌号
     * @param groupId 集团id
     * @return
     */
    @GetMapping("/query/accident/follower")
    Message<AccidentFollowerResult> queryByPlate(@RequestParam("plateNo") String plateNo, @RequestParam("groupId") Long groupId);

    /**
     * 查询[门店/人员] 进站率目标
     *
     * @param inboundRateTargetReq 参数 门店/人员不能同时为空
     * @return 进站目标信息
     * @since 1.3.0
     */
    @PostMapping("/query/inbound_rate_target")
    Message<InboundRateTargetResult> queryInboundRateTarget(@Valid @RequestBody InboundRateTargetReq inboundRateTargetReq);

    /**
     * 分配保有客
     * <p> 1.传入的档案[vin]必须有专属人员 </p>
     * <p> 2.只有传入的staffId不等于档案专属人员id </p>
     * <p> 3.传入的staffId对应人员必须有服务顾问角色 </p>
     * <p> 4.档案原本专属人员无服务顾问角色 </p>
     * <p> 只有当以上条件才会执行分配逻辑！只有当以上条件才会执行分配逻辑！只有当以上条件才会执行分配逻辑！</p>
     * <p> 接口调用无论是否执行分配逻辑都是返回成功，只有当分配抛出异常了才会失败，失败的情况需要重试 </p>
     *
     * @param distributionReq 参数信息
     * @return boolean
     * @since 1.3.1
     */
    @PostMapping("/distribution/customer")
    Message<Void> distribution(@Valid @RequestBody CustomerDistributionReq distributionReq);

    /**
     * <p>终止档案贷款期状态</p>
     * <p>提前还清贷款的情况</p>
     *
     * @param vin     车架号
     * @param groupId 集团id
     * @since 1.3.2
     */
    @GetMapping("/stop/customer/loan")
    Message<Void> stopCustomerLoan(@RequestParam("vin") String vin, @RequestParam("groupId") Long groupId);
}
