package cn.fw.valhalla.service.bus;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.ThreadPoolUtil;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerReachLog;
import cn.fw.valhalla.domain.enums.ReachInvalidReasonEnum;
import cn.fw.valhalla.domain.enums.SettingTypeEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.sdk.param.InboundRateTargetReq;
import cn.fw.valhalla.sdk.param.ReachLogReq;
import cn.fw.valhalla.sdk.result.InboundRateTargetResult;
import cn.fw.valhalla.service.bus.setting.strategy.impl.GeneralSetting;
import cn.fw.valhalla.service.data.CustomerReachLogService;
import cn.fw.valhalla.service.data.CustomerService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.Date;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : ReachLogBizService
 * @description : 进站记录BIZ
 * @date: 2020-11-11 17:48
 */
@Service
@RequiredArgsConstructor
public class ReachLogBizService {
    private final CustomerService customerService;
    private final CustomerReachLogService customerReachLogService;
    private final EhrRpcService ehrRpcService;
    private final GeneralSetting generalSetting;
    private final StringRedisTemplate stringRedisTemplate;

    @Value("${spring.cache.custom.global-prefix}:reach-log:cache")
    @Getter
    private String reachLogKey;

    @Transactional(rollbackFor = Exception.class)
    public void saveReachLog(ReachLogReq reachLogReq) {
        BV.isTrue(reachLogReq.checkMileage(), () -> "里程不能为空");
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(getReachLogKey());
        String jsonString = JSONObject.toJSONString(reachLogReq);
        listOps.rightPush(jsonString);
    }

    /**
     * 保存进站数据(新版)
     *
     * @param reachData
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveReachLog(String reachData) {
        ReachLogReq reachLogReq = JSONObject.parseObject(reachData, ReachLogReq.class);
        Long customerId = reachLogReq.getCustomerId();
        Customer customer = customerService.getById(customerId);
        BV.notNull(customer, () -> "Customer not found");
        Date arrivalTime = reachLogReq.getArrivalTime();
        Long shopId = reachLogReq.getShopId();
        LocalDate localDate = DateUtil.date2LocalDate(arrivalTime);
        YearMonth month = YearMonth.from(localDate);
        Long adviserId = customer.getAdviserId();
        Long reachAdviserId = reachLogReq.getAdviserId();

        CustomerReachLog reachLog = new CustomerReachLog();
        reachLog.setCustomerId(customerId);
        reachLog.setFrameNo(customer.getFrameNo());
        reachLog.setOrderNo(reachLogReq.getOrderNo());
        reachLog.setShopId(reachLogReq.getShopId());
        reachLog.setAdviserId(reachAdviserId);
        reachLog.setArrivalTime(reachLogReq.getArrivalTime());
        reachLog.setDataMonth(YearMonth.from(DateUtil.date2LocalDate(reachLogReq.getArrivalTime())));
        reachLog.setGroupId(customer.getGroupId());
        reachLog.setExcludeStatistics(Boolean.FALSE);
        UserInfoDTO user = ehrRpcService.user(reachAdviserId);
        if (Objects.nonNull(user)) {
            reachLog.setAdviserName(user.getUserName());
        }

        if (!Objects.equals(adviserId, reachAdviserId)) {
            reachLog.setValid(Boolean.FALSE);
            reachLog.setReason(ReachInvalidReasonEnum.NATURAL);
            reachLog.setExcludeStatistics(Boolean.TRUE);
        } else {
            boolean exist = currentMonthExist(customer.getFrameNo(), month, shopId);
            if (!exist) {
                reachLog.setValid(Boolean.TRUE);
                reachLog.setReason(ReachInvalidReasonEnum.INAPPLICABLE);
            } else {
                reachLog.setValid(Boolean.FALSE);
                reachLog.setReason(ReachInvalidReasonEnum.DUPLICATE);
            }
        }

        customerReachLogService.save(reachLog);

        int count = Objects.nonNull(customer.getArrivalCount()) ? customer.getArrivalCount() : 0;
        customer.setArrivalTime(arrivalTime);
        customer.setArrivalCount(count + 1);
        customer.setLastArrivalShop(shopId);
        Integer arrivalMileage = reachLogReq.getArrivalMileage();
        Integer fuelMileage = reachLogReq.getFuelMileage();
        Integer electricMileage = reachLogReq.getElectricMileage();
        if (Objects.isNull(fuelMileage)) {
            fuelMileage = 0;
            if (Objects.nonNull(arrivalMileage)) {
                fuelMileage = arrivalMileage;
            }
        }
        if (Objects.isNull(electricMileage)) {
            electricMileage = 0;
        }
        customer.setCurrentMileage(fuelMileage + electricMileage);
        customerService.updateById(customer);

        if (Boolean.TRUE.equals(reachLog.getValid())) {
            customerReachLogService.undoExclusion(customerId, shopId, reachAdviserId, reachLog.getDataMonth());
        }
    }


    /**
     * 查询进站率目标
     *
     * @param inboundRateTargetReq
     * @return
     */
    public InboundRateTargetResult queryInboundRateTarget(final InboundRateTargetReq inboundRateTargetReq) {
        final InboundRateTargetResult result = new InboundRateTargetResult();
        ExecutorService executor = ThreadPoolUtil.getInstance().getExecutor();
        CompletableFuture<Void> future1 = CompletableFuture.runAsync(() -> queryInboundTarget(result, inboundRateTargetReq), executor);
        CompletableFuture<Void> future2 = CompletableFuture.runAsync(() -> queryInboundCount(result, inboundRateTargetReq), executor);
        CompletableFuture<Void> future3 = CompletableFuture.runAsync(() -> queryCustomerCount(result, inboundRateTargetReq), executor);
        CompletableFuture.allOf(future1, future2, future3).join();
        BigDecimal inbound_rate = BigDecimal.valueOf(result.getInboundNum()).divide(BigDecimal.valueOf(result.getCustomerNum()), 2, RoundingMode.HALF_UP);
        result.setInboundRate(inbound_rate);
        int targetNum = BigDecimal.valueOf(result.getCustomerNum()).multiply(result.getTargetRate()).divide(BigDecimal.ONE, 0, RoundingMode.HALF_UP).intValue();
        result.setTargetNum(targetNum);
        return result;
    }

    private void queryInboundTarget(final InboundRateTargetResult result, final InboundRateTargetReq inboundRateTargetReq) {
        SettingVO settingVO = generalSetting.getBySettingType(SettingTypeEnum.PIT_STOP_RATIO, inboundRateTargetReq.getGroupId(), null);
        int detailValue = Optional.ofNullable(settingVO).map(SettingVO::getDetailValue).orElse(0);
        BigDecimal rate = BigDecimal.valueOf(detailValue).divide(BigDecimal.valueOf(10000), 2, RoundingMode.HALF_UP);
        result.setTargetRate(rate);
    }


    private void queryInboundCount(final InboundRateTargetResult result, final InboundRateTargetReq inboundRateTargetReq) {
        LocalDate mothFirst = YearMonth.now().atDay(1);
        Date date = DateUtil.localDate2Date(mothFirst);
        int inbound_count = customerReachLogService.count(Wrappers.<CustomerReachLog>lambdaQuery()
                .ge(CustomerReachLog::getArrivalTime, date)
                .le(CustomerReachLog::getArrivalTime, new Date())
                .eq(CustomerReachLog::getGroupId, inboundRateTargetReq.getGroupId())
                .in(!CollectionUtils.isEmpty(inboundRateTargetReq.getShopIds()), CustomerReachLog::getShopId, inboundRateTargetReq.getShopIds())
                .in(!CollectionUtils.isEmpty(inboundRateTargetReq.getStaffUserIds()), CustomerReachLog::getAdviserId, inboundRateTargetReq.getStaffUserIds())
        );

        result.setInboundNum(inbound_count);
    }

    private void queryCustomerCount(final InboundRateTargetResult result, final InboundRateTargetReq inboundRateTargetReq) {
        int customer_count = customerService.count(Wrappers.<Customer>lambdaQuery()
                .ne(Customer::getTemporary, Boolean.TRUE)
                .eq(Customer::getYn, Boolean.TRUE)
                .eq(Customer::getGroupId, inboundRateTargetReq.getGroupId())
                .in(!CollectionUtils.isEmpty(inboundRateTargetReq.getShopIds()), Customer::getShopId, inboundRateTargetReq.getShopIds())
                .in(!CollectionUtils.isEmpty(inboundRateTargetReq.getStaffUserIds()), Customer::getAdviserId, inboundRateTargetReq.getStaffUserIds())
        );

        result.setCustomerNum(customer_count);
    }

    /**
     * 进站数据前置处理
     *
     * @param vin
     * @param month
     * @return
     */
    private void preDeal(String vin, YearMonth month, Long shopId) {

    }

    /**
     * 本月是否已经记录进站信息
     *
     * @param vin
     * @param month
     * @return
     */
    private boolean currentMonthExist(String vin, YearMonth month, Long shopId) {
        return customerReachLogService.count(Wrappers.<CustomerReachLog>lambdaQuery()
                .eq(CustomerReachLog::getFrameNo, vin)
                .eq(CustomerReachLog::getDataMonth, month)
                .eq(CustomerReachLog::getShopId, shopId)
                .eq(CustomerReachLog::getValid, Boolean.TRUE)
        ) > 0;
    }
}
