package cn.fw.valhalla.service.bus.cust;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.component.producer.CustomerChangeProducer;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.db.customer.CustomerLoanInfo;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.db.pool.PublicPool;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.db.pub.PubCluePool;
import cn.fw.valhalla.domain.dto.CustomerDetailDto;
import cn.fw.valhalla.domain.enums.ClueStatusEnum;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.member.MemberRpcService;
import cn.fw.valhalla.rpc.member.dto.MemberUserDTO;
import cn.fw.valhalla.sdk.result.CustomerInfoDto;
import cn.fw.valhalla.service.data.*;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * 通用调试服务
 *
 * @author : kurisu
 * @version : 1.0
 * @className : CommonBizService
 * @description : 通用调试服务
 * @date : 2023-05-30 14:25
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class CommonBizService {
    private final CustomerService customerService;
    private final CustomerBaseInfoService customerBaseInfoService;
    private final CustomerChangeProducer customerChangeProducer;
    private final EhrRpcService ehrRpcService;
    private final CustomerLoanInfoService customerLoanInfoService;
    private final MemberRpcService memberRpcService;
    private final PublicPoolService publicPoolService;
    private final StammkundePoolService stammkundePoolService;
    private final FollowClueService followClueService;
    private final ClueTaskService clueTaskService;
    private final FollowNoticeRecordService followNoticeRecordService;
    private final PubCluePoolService pubCluePoolService;

    private final AtomicLong cursor = new AtomicLong(0L);


    public void scyncAllCustomer(Long groupId) {
        while (true) {
            long id = cursor.get();
            List<Customer> list = customerService.list(Wrappers.<Customer>lambdaQuery()
                    .gt(Customer::getId, id)
                    .eq(Customer::getYn, Boolean.TRUE)
                    .eq(Customer::getGroupId, groupId)
                    .orderByAsc(Customer::getId)
                    .last("limit 1500")
            );

            if (CollectionUtils.isEmpty(list) || list.size() < 1500) {
                cursor.set(0L);
                return;
            }
            Customer lastCustomer = list.get(list.size() - 1);
            long lastId = lastCustomer.getId();
            cursor.compareAndSet(id, lastId);
            List<Long> idList = list.stream().map(Customer::getId).collect(Collectors.toList());
            customerChangeProducer.prepareSendCustomerChangeMq(idList);
        }
    }

    public void sendNotice(Long id) {
        Customer customer = customerService.queryById(id);
        if (Objects.isNull(customer)) {
            return;
        }
        CustomerBaseInfo baseInfo = customerBaseInfoService.queryById(customer.getBaseId());
        CustomerDetailDto detailDto = renderDto(customer, baseInfo);
        CustomerInfoDto dto = new CustomerInfoDto();
        BeanUtils.copyProperties(detailDto, dto);
        if (Objects.isNull(dto.getShopId())) {
            PublicPool publicPool = publicPoolService.queryByVin(customer.getFrameNo(), customer.getGroupId());
            if (Objects.nonNull(publicPool)) {
                dto.setFromPublic(Boolean.TRUE);
                dto.setShopId(publicPool.getShopId());
            }
        }
        customerChangeProducer.send(dto);
    }

    @Transactional(rollbackFor = Exception.class)
    public void rollback(String vin, Long groupId) {
        Customer customer = customerService.getOne(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getFrameNo, vin)
                .eq(Customer::getYn, Boolean.TRUE)
                .eq(Customer::getGroupId, groupId)
                .last(" limit 1 ")
        );
        if (Objects.isNull(customer)) {
            return;
        }
        LocalDateTime createTime = DateUtil.date2LocalDateTime(customer.getCreateTime());
        boolean after = createTime.isAfter(LocalDateTime.now().minusDays(7L));
        BV.isTrue(after, () -> "档案创建已超过7天，请联系开发人员手动处理");
        List<FollowClue> list = followClueService.list(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getVin, vin)
                .eq(FollowClue::getGroupId, groupId)
        );
        Long customerId = customer.getId();
        List<PubCluePool> pubCluePools = pubCluePoolService.list(Wrappers.<PubCluePool>lambdaQuery()
                .eq(PubCluePool::getVin, vin)
                .eq(PubCluePool::getGroupId, groupId)
        );
        BV.isEmpty(pubCluePools, () -> "存在已开始的跟进任务，无法回滚档案");
        if (!CollectionUtils.isEmpty(list)) {
            List<Long> clueIds = new ArrayList<>();
            for (FollowClue cluePool : list) {
                boolean isWaiting = ClueStatusEnum.WAITING.equals(cluePool.getClueState());
                BV.isTrue(isWaiting, () -> "存在已开始的跟进任务，无法回滚档案");
                clueIds.add(cluePool.getId());
            }
            clueTaskService.remove(Wrappers.<ClueTask>lambdaQuery()
                    .eq(ClueTask::getGroupId, groupId)
                    .in(ClueTask::getType, FollowTypeEnum.FM, FollowTypeEnum.RM, FollowTypeEnum.AC, FollowTypeEnum.IR)
                    .in(ClueTask::getClueId, clueIds)
            );
            followNoticeRecordService.removeByClueIds(clueIds);
            followClueService.removeByIds(clueIds);
        }
        publicPoolService.removeByCustomerId(customerId, groupId);
        List<StammkundePool> stammkundePoolList = stammkundePoolService.list(Wrappers.<StammkundePool>lambdaQuery()
                .eq(StammkundePool::getCustomerId, customerId)
                .eq(StammkundePool::getGroupId, groupId)
        );
        if (!CollectionUtils.isEmpty(stammkundePoolList)) {
            stammkundePoolService.removeByCustomerId(customerId, groupId);
        }
        customerService.removeById(customerId);
    }

    private CustomerDetailDto renderDto(@NotNull Customer customer, CustomerBaseInfo baseInfo) {
        CustomerDetailDto dto = new CustomerDetailDto();
        if (Objects.nonNull(baseInfo)) {
            dto.setMemberId(baseInfo.getMemberId());
            dto.setName(baseInfo.getName());
            dto.setCompanyName(baseInfo.getCompanyName());

            MemberUserDTO user = memberRpcService.user(baseInfo.getMemberId());
            if (Objects.nonNull(user)) {
                dto.setMobile(user.getPhone());
                dto.setRegion(memberRpcService.attribution(user.getPhone()));
            }

            dto.setCusType(Objects.isNull(baseInfo.getCusType()) ? 0 : baseInfo.getCusType().getValue());
            dto.setIdCode(baseInfo.getIdCode());
            dto.setGender(baseInfo.getGender().getValue());
            dto.setContact(baseInfo.getContact());
            dto.setBirthday(baseInfo.getBirthday());
            dto.setAddress(baseInfo.getAddress());
            dto.setLat(baseInfo.getLat());
            dto.setLng(baseInfo.getLng());
            dto.setHobby(baseInfo.getHobby());
            dto.setCityCode(baseInfo.getCityCode());
            dto.setCityName(baseInfo.getCityName());
        }
        CustomerLoanInfo loanInfo = customerLoanInfoService.getOne(Wrappers.<CustomerLoanInfo>lambdaQuery()
                .eq(CustomerLoanInfo::getGroupId, customer.getGroupId())
                .eq(CustomerLoanInfo::getFrameNo, customer.getFrameNo())
                .eq(CustomerLoanInfo::getYn, Boolean.TRUE)
                .last(" limit 1")
        );
        if (Objects.nonNull(loanInfo)) {
            dto.setLoanCustomer(LocalDate.now().isBefore(DateUtil.date2LocalDate(loanInfo.getExpires())));
            dto.setExpires(loanInfo.getExpires());
            dto.setPeriods(loanInfo.getPeriods());
        }
        dto.setId(customer.getId());
        dto.setPlateNo(customer.getPlateNo());
        dto.setBrandId(customer.getBrandId());
        dto.setBrandName(customer.getBrandName());
        dto.setSeriesId(customer.getSeriesId());
        dto.setSeriesName(customer.getSeriesName());
        dto.setSpecId(customer.getSpecId());
        dto.setFrameNo(customer.getFrameNo());
        dto.setSpecName(customer.getSpecName());
        dto.setCarImage(customer.getCarImage());
        dto.setTags(customer.getTags());
        dto.setCusLevel(customer.getCusLevel());
        dto.setCurrentMileage(customer.getCurrentMileage());
        dto.setInsuranceExpires(customer.getInsuranceExpires());
        dto.setArrivalTime(customer.getArrivalTime());
        dto.setAdviserId(customer.getAdviserId());
        dto.setArrivalCount(customer.getArrivalCount());
        dto.setTemporary(customer.getTemporary());
        UserInfoDTO user = ehrRpcService.user(customer.getAdviserId());
        if (Objects.nonNull(user)) {
            dto.setAdviserName(user.getUserName());
        }
        dto.setEngineNo(customer.getEngineNo());
        dto.setRegDate(customer.getRegDate());
        dto.setSpecCode(customer.getSpecCode());
        dto.setBuyDate(customer.getBuyDate());
        dto.setUseType(Objects.isNull(customer.getUseType()) ? null : customer.getUseType().getValue());
        dto.setCreateTime(customer.getCreateTime());
        dto.setGroupId(customer.getGroupId());
        dto.setVehicleLicense(customer.getVehicleLicense());
        dto.setShopId(customer.getShopId());
        dto.setLastArrivalShop(customer.getLastArrivalShop());
        dto.setBuyPrice(customer.getBuyPrice());
        dto.setSource(Objects.isNull(customer.getSource()) ? null : customer.getSource().getValue());
        dto.setYn(customer.getYn());
        return dto;
    }
}
